## vim:ts=4:et:nowrap
##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
##---------------------------------------------------------------------------##


# imports
import sys

# PySol imports
if sys.modules.has_key('pysoltk'):                                  #bundle#
    from plugin import Plugin, registerGame, registerPlugin         #bundle#
    from stack import *                                             #bundle#
    from game import Game                                           #bundle#
    from hint import Hint, DefaultHint                              #bundle#
    from pysoltk import MfxCanvasText                               #bundle#
    from seahaventowers import SeahavenTowers_Hint, SeahavenTowers_RowStack #bundle#


# /***********************************************************************
# // Eight Off
# ************************************************************************/

class EightOff(Plugin):

    #
    # game layout
    #

    def createGame(self):
        images = self.app.images
        XMARGIN = 10
        YMARGIN = 8
        XSPACE = images.CARDW + XMARGIN
        YSPACE = images.CARDH + YMARGIN

        # set window
        # (piles up to 12 cards are playable without overlap in default window size)
        h = max(2*YSPACE, YSPACE+(12-1)*images.FACE_UP_YOFFSET)
        self.setSize(8*XSPACE+XMARGIN, YMARGIN + YSPACE + h + YSPACE)

        # create stacks
        x = XMARGIN + 2 * XSPACE
        y = YMARGIN
        for i in range(4):
            self.s.foundations.append(SuitFoundationStack(x, y, self, i))
            x = x + XSPACE
        x = XMARGIN
        y = y + YSPACE
        for i in range(8):
            self.s.rows.append(SeahavenTowers_RowStack(x, y, self))
            x = x + XSPACE
        x = XMARGIN
        y = self.height - YSPACE
        for i in range(8):
            self.s.reserves.append(ReserveStack(x, y, self))
            x = x + XSPACE
        self.setRegion(self.s.reserves, [ -999, y - images.CARDH / 2, 999999, 999999 ])
        self.s.talon = InvisibleTalonStack(XMARGIN, YMARGIN, self)

        # create cards
        self.cards = self.s.talon.createCards(1)

        # define stack-groups
        self.sg.openstacks = self.s.foundations + self.s.rows + self.s.reserves
        self.sg.talonstacks = [self.s.talon]
        self.sg.dropstacks = self.s.rows + self.s.reserves
        self.sg.reservestacks = self.s.reserves

    #
    # game overrides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 1 * 52
        self.moves.state = self.S_DEAL
        for i in range(5):
            self.s.talon.dealRow(frames=0)
        self.s.talon.dealRow()
        r = self.s.reserves
        self.s.talon.dealRow(rows=[r[0],r[2],r[4],r[6]])
        assert len(self.s.talon.cards) == 0

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.suit == card2.suit and
                (card1.rank + 1 == card2.rank or card2.rank + 1 == card1.rank))

    def getHintClass(self):
        return SeahavenTowers_Hint

    def getStrictness(self):
        # needed for SeahavenTowers_RowStack
        return 1


# register the plugin: id, class, name, version and minimum PySol version
registerPlugin(258, EightOff, "Eight Off", "1.00", "2.11")

